<?PHP /* /mod/semacode/lib.php (c)2007, Andreas Forstinger, www.mobilemoodle.org */

global $CFG;

/// define default configuration data

    if (!isset($CFG->semacode_supported)) { // identifier, used by other modules to check availability
        set_config('semacode_supported', true);
    }
    if (!isset($CFG->semacode_java_cmd)) {
        set_config('semacode_java_cmd', 'java');
    }
    if (!isset($CFG->semacode_file_type)) {
        set_config('semacode_file_type', 'png');
    }
    if (!isset($CFG->semacode_tag_width)) {
        set_config('semacode_tag_width', 300);
    }
    if (!isset($CFG->semacode_path)) {
        set_config('semacode_path', 'semacode/');
    }
    if (!isset($CFG->semacode_link_url)) {
        set_config('semacode_link_url', '/tag.php?id='); // link relative to moodle root
    }
    if (!isset($CFG->semacode_creator)) {
        set_config('semacode_creator', $CFG->dirroot . '/mod/semacode/creator/semacode_tag.jar');
    }

function semacode_add_instance($form) {
    global $CFG;

    // prepare data object
    $semacode = new object;
    $semacode->course = clean_param($form->course,PARAM_INT);
    $semacode->name = clean_param($form->name,PARAM_NOTAGS);
    $semacode->description = clean_param($form->description,PARAM_TEXT);
    $semacode->timecreated = time();
    $semacode->lastmodified = time();

    // set rest of fields depending on type
    // u - url, c - course, a - activity, l - location

    // tag has latitude and longitude set -> type 'l'
    if(($form->url == 'http://') && empty($form->trgt_course) && empty($form->trgt_activity) && !empty($form->latitude) && !empty($form->longitude)){
        $semacode->type = 'l';
        $semacode->latitude = $form->latitude;
        $semacode->longitude = $form->longitude;
    }
    // tag has only url set -> type 'u'
    else if(!empty($form->url)&& empty($form->trgt_course) && empty($form->trgt_activity)&& empty($form->latitude) && empty($form->longitude)){
        $semacode->type = 'u';
        $semacode->url = clean_param($form->url,PARAM_URL);
    }
    // tag has only course set -> type 'c'
    else if(($form->url == 'http://') && !empty($form->trgt_course) && empty($form->trgt_activity)){
        $semacode->type = 'c';
        $semacode->target = clean_param($form->trgt_course,PARAM_INT);
    }
    // tag has only activity set -> type 'a'
    else if(($form->url == 'http://') && empty($form->trgt_course) && !empty($form->trgt_activity)){
        $semacode->type = 'a';
        $semacode->target = clean_param($form->trgt_activity,PARAM_INT);
    }

    // error
    else{
        unset($semacode);
        return false;
    }

    // we temporarily store the entry to db
    $semacode->id = insert_record('semacode',$semacode, true);
    if(empty($semacode->id)){return false;}

    // now we fire up creation of the tag
    if(!semacode_create_tag($semacode)){
        // an error occured, so delete temporary file and db record
        semacode_delete_tag($semacode->id);
        delete_records('semacode', 'id', $semacode->id);
        // show error page
        print_error('error_creation','semacode',$CFG->wwwroot . "/course/view.php?id=$semacode->course");
        return false;
    }
    else{
        // everything went right
        if(!update_record('semacode',$semacode)){
            return false;
        }
        else{
            return $semacode->id;
        }
    }
}

function semacode_update_instance($form) {
    global $CFG;

    /// get existing record
    $semacode = get_record('semacode','id',$form->instance);
    $oldurl = $semacode->url;

    // set main values
    $semacode->name = clean_param($form->name,PARAM_NOTAGS);
    $semacode->description = clean_param($form->description,PARAM_TEXT);
    $semacode->lastmodified = time();

    // tag has latitude and longitude set -> type 'l'
    if(($form->url == 'http://') && empty($form->trgt_course) && empty($form->trgt_activity) && !empty($form->latitude) && !empty($form->longitude)){
        $semacode->type = 'l';
        $semacode->latitude = $form->latitude;
        $semacode->longitude = $form->longitude;
    }
    // tag has only url set -> type 'u'
    else if(!empty($form->url)&& empty($form->trgt_course) && empty($form->trgt_activity)){
        $semacode->type = 'u';
        $semacode->url = clean_param($form->url,PARAM_URL);
    }
    // tag has only course set -> type 'c'
    else if(($form->url == 'http://') && !empty($form->trgt_course) && empty($form->trgt_activity)){
        $semacode->type = 'c';
        $semacode->target = clean_param($form->trgt_course,PARAM_INT);
    }
    // tag has only activity set -> type 'a'
    else if(($form->url == 'http://') && empty($form->trgt_course) && !empty($form->trgt_activity)){
        $semacode->type = 'a';
        $semacode->target = clean_param($form->trgt_activity,PARAM_INT);
    }


    /// recreate the tag if url has changed
    if(($semacode->type == 'u' && $oldurl != $semacode->url) || ("$CFG->wwwroot . $CFG->semacode_link_url . $semacode->id" != $semacode->url)){

        // make sure to delete old file (there have been problems with overwriting)
        semacode_delete_tag($semacode->id);

        // recreate
        if(!$tagfile = semacode_create_tag($semacode)){
            return false;
        }

        else {
              // everything went fine until here so finally update db record
            return update_record('semacode',$semacode);
        }
    }
}

function semacode_delete_instance($id) {
/// Given an ID of an instance of this module,
/// this function will permanently delete the instance
/// and any data that depends on it.
    global $CFG;

    if (! $semacode = get_record('semacode', 'id', "$id")) {
        return false;
    }

    $fileresult = $dbresult = false;

    $fileresult = semacode_delete_tag($semacode->id);

    if($fileresult){
        // also delete db record
        $dbresult = delete_records('semacode', 'id', "$semacode->id");
    }

    return $fileresult && $dbresult;
}

/**
 *
 */
function semacode_create_location_tag($longitude,$latitude,$name='auto', $description = 'automatically created location tag'){

    global $CFG,$COURSE;

    // prepare dataobject
    $semacode =  new stdClass();
    $semacode->course = $COURSE->id;
    $semacode->name = $name;
    $semacode->type = 'l';
    $semacode->longitude = $longitude;
    $semacode->latitude = $latitude;
    $semacode->timecreated = $semacode->lastmodified = time();
    $semacode->description = $description;

    // temporarily store to db
    $success = insert_record('semacode',$semacode);

    if(!empty($success)){
        $semacode->id = $success;
        // create tag
        semacode_create_tag($semacode);
    }

    // update record (insert url)
    update_record('semacode',$semacode);

    // return id for linking
    return $semacode->id;

}

function semacode_update_location_tag($id,$longitude,$latitude,$name='auto'){
    global $CFG,$COURSE;

    if(!$semacode = get_record('semacode','id',$id)){
        return false;
    }
    $semacode->name = $name;
    $semacode->type = 'l';
    $semacode->longitude = $longitude;
    $semacode->latitude = $latitude;
    $semacode->lastmodified = time();

    // recreate the tag if the url has changed
    if("$CFG->wwwroot . $CFG->semacode_link_url . $semacode->id" != $semacode->url){
        // make sure to delete old file (there have been problems with overwriting)
        semacode_delete_tag($semacode->id);
        // recreate the tag
        semacode_create_tag($semacode);
    }

    // update db entry
    $success = update_record('semacode',$semacode);

    // return id for linking
    return $semacode->id;
}

function semacode_delete_location_tag($id){

    if(!$semacode = get_record('semacode','id',$id)){
        return false;
    }

    $fileresult = $dbresult = false;
    $fileresult = semacode_delete_tag($semacode->id);
    if($fileresult){
        // also delete db record
        $dbresult = delete_records('semacode', 'id', "$semacode->id");
    }
    return $fileresult && $dbresult;
}

/**
 * This function creates a semacode tag according to the given data
 * the tag is stored using the id as filename
 * @return true if everything went fine, false if an error occured
 */
function semacode_create_tag(&$semacode){
    global $CFG;
    // first make sure data directories exist
    $directory = make_upload_directory($CFG->semacode_path);

    // we create the tag using id as the filename
    $filename = $directory . $semacode->id . '.' . $CFG->semacode_file_type;

    // define url for tags linking into moodle site
    if($semacode->type != 'u'){
        $semacode->url = $CFG->wwwroot . $CFG->semacode_link_url . $semacode->id;
    }

    // invoke semacode sdk tool (www.semacode.org)
    $output = $error = null;
    $command = $CFG->semacode_java_cmd . ' -Djava.awt.headless=true -jar ' . $CFG->semacode_creator . ' -w ' . $CFG->semacode_tag_width .  ' -u "' . $semacode->url . '" -f ' . $filename;
    // attention - this might lead to errors on servers in safe mode
    exec($command, $output, $error);

    // check return code - if > 0 an error occured
    if($error > 0 ) {
        // log error
        add_to_log($semacode->course, 'semacode', 'creation', '', "failed ($error)", '');
        return false;
    }
    else{
        return true;
    }
}

/**
 * This function deletes the given tag file
 */
function semacode_delete_tag($id){
    global $CFG;
    // first make sure data directory exists
    $directory = make_upload_directory($CFG->semacode_path);
    $file = $directory . $id . '.' . $CFG->semacode_file_type;

    $success = false;
    if(file_exists($file)){
        $success = unlink($file);
    }
    return $success;
}

function semacode_user_outline($course, $user, $mod, $semacode) {
/// Return a small object with summary information about what a
/// user has done with a given particular instance of this module
/// Used for user activity reports.
/// $return->time = the time they did it
/// $return->info = a short text description

    return null;
}

function semacode_user_complete($course, $user, $mod, $semacode) {
/// Print a detailed representation of what a  user has done with
/// a given particular instance of this module, for user activity reports.

    return true;
}

function semacode_print_recent_activity($course, $isteacher, $timestart) {
/// Given a course and a time, this module should find recent activity
/// that has occurred in semacode activities and print it out.
/// Return true if there was output, or false is there was none.

    return false;  //  True if anything was printed, otherwise false
}

function semacode_cron () {
/// Function to be run periodically according to the moodle cron
/// This function searches for things that need to be done, such
/// as sending out mail, toggling flags etc ...

}

function semacode_grades($semacodeid) {
/// Must return an array of grades for a given instance of this module,
/// indexed by user.  It also returns a maximum allowed grade.
///
///    $return->grades = array of grades;
///    $return->maxgrade = maximum allowed grade;
///
///    return $return;

   return NULL;
}

function semacode_get_participants($semacodeid) {
//Must return an array of user records (all data) who are participants
//for a given instance of semacode. Must include every user involved
//in the instance, independient of his role (student, teacher, admin...)
//See other modules as example.

    return false;
}

function semacode_scale_used ($semacodeid,$scaleid) {
//This function returns if a scale is being used by one semacode
//it it has support for grading and scales. Commented code should be
//modified if necessary. See forum, glossary or journal modules
//as reference.

    $return = false;

    //$rec = get_record("semacode","id","$semacodeid","scale","-$scaleid");
    //
    //if (!empty($rec)  && !empty($scaleid)) {
    //    $return = true;
    //}

    return $return;
}

/**
 * Given a course object, this function will clean up anything that
 * would be leftover after all the instances were deleted.
 * @param object $course an object representing the course that is being deleted
 * @param boolean $feedback to specify if the process must output a summary of its work
 * @return boolean
 */
function semacode_delete_course($course,$feedback=true) {

    // get all instances in the given course
    $semacodes = get_records('semacode','course',$course->id);

    // manually delete each instance
    foreach ($semacodes as $semacode){
        semacode_delete_instance($semacode->id);
    }

    return true;
}
//////////////////////////////////////////////////////////////////////////////////////
/// Any other semacode functions go here.  Each of them must have a name that
/// starts with semacode_


?>