<?php /* /momo/lib/lib_msg_local.php, (c) 2008, Andreas Forstinger, www.mobilemoodle.org */
// define the global variable

global $pushDB;

require_once($CFG->libdir .'/adodb/adodb.inc.php'); // Database access functions

function momo_pushdb_connect(){
    /// connect to MLE messaging server database
    global $CFG, $pushDB;

    $pushDB = ADONewConnection($CFG->push_dbtype);

    $pushDB->debug = false;

    // (taken from lib/setup.php)
    // See MDL-6760 for why this is necessary. In Moodle 1.8, once we start using NULLs properly,
    // we probably want to change this value to ''.
    $pushDB->null2null = 'A long random string that will never, ever match something we want to insert into the database, I hope. \'';
    //error_reporting(0); // Hide errors
    if (!isset ($CFG->push_dbpersist) or !empty ($CFG->push_dbpersist)) { // Use persistent connection (default)
        // $dbconnected = $pushDB->PConnect($CFG->push_dbhost, $CFG->push_dbuser, $CFG->push_dbpass, $CFG->push_dbname);
     	   $dbconnected = $pushDB->PConnect($CFG->push_dbhost);
    } else { // Use single connection
        //$dbconnected = $pushDB->Connect($CFG->push_dbhost, $CFG->push_dbuser, $CFG->push_dbpass, $CFG->push_dbname);
          $dbconnected = $pushDB->Connect($CFG->push_dbhost);
    }
    
    if (!$dbconnected) {
        // connection failed
    	print("!connection failed!");
        return false;
    }
    else{
        /// Forcing ASSOC mode for ADOdb (some DBs default to FETCH_BOTH)
        $pushDB->SetFetchMode(ADODB_FETCH_ASSOC);
        // restore error report level
        error_reporting(E_ALL); // Show errors from now on.
        return true;
    }
}


/**
 * This method executes the given query and returns the resultset
 * @return resultset
 */
function _momo_msg_execute($query) {
    global $pushDB;

    if(!isset($pushDB)){
        momo_pushdb_connect();
    }
    // execute query
    $result = $pushDB->Execute($query);
//    print_r($result);
    return $result;
}

/**
 * This function returns the SINGLE result of the given query as an object
 */
function momo_msg_get_record($query) {
    // get record set
    if (!$result = _momo_msg_execute($query)) {
        return false;
    }
    // check record set
    $recordcount = $result->RecordCount();
    if ($recordcount == 1) { // exactly one record was found
        return (object) $result->fields;
    } else {
        return false;
    }
}

/**
 * This function returns the MULTIPLE results
 *  of the given query as objects
 */
function momo_msg_get_records($query) {
    // get record set
    if (!$result = _momo_msg_execute($query)) {
        return false;
    }
    // return result as array of objects
    return recordset_to_array($result);
}

/**
 * This function returns an array of users currently connected via MOMO
 * @return array of user objects or false
 */
function momo_msg_get_online_users() {
    $query = 'select LOWER(username) as username from onlineclients';
    return momo_msg_get_records($query);
}

/**
 * This function returns the login data for the requested username
 * @return object containing login data or false
 */
function momo_msg_get_login_data($username) {
    $query = "select id, username, servername, pass from onlineclients where lower(username) = '" . strtolower($username) . "'";
    return momo_msg_get_record($query);
}

/**
 * This function is used to push a message notice to the given user if he is currently online
 * via MOMO
 */
function momo_msg_push_message_notice($userid, $newmsg,$online = true) {
    // get username for this id
    if (!$username = get_field('user', 'username', 'id', $userid)) {
        return false;
    }
    // check if this user is connected
    if (!$recipient = momo_msg_get_login_data($username)) {
        return false;
    }
    else{
    	//print_r($recipient);
        // get the number of messages waiting for this user
        $messages = count_records('message', 'useridto', $userid);
        //print($messages);
        if (!empty ($messages)) {
            _momo_msg_push_message($recipient->username, $recipient->pass, $recipient->servername, $messages, 'momo', $newmsg?'msgnew':'msg', $online);
        }
    }
}

/**
 * This function is used to push a notice to the given user (Live! Session Block)
 */
function momo_msg_push_session_notice($userid, $online = true) {
    // get username for this id
    if (!$username = get_field('user', 'username', 'id', $userid)) {
        return false;
    }
    // check if this user is connected
    if ($recipient = momo_msg_get_login_data($username)) {
        _momo_msg_push_message($recipient->username, $recipient->pass, $recipient->servername, 'x', 'momo', 'event', $online);
    }
}

/**
 * This function is used internally to push messages to a mobile client
 */
function _momo_msg_push_message($username, $pass, $server, $msg = '.', $dest, $type, $online = true) {
    global $CFG;
    if (!empty ($online))
        $online2 = "true";
    else
        $online2 = "false";
    // mind to manually get next id from postgres sequence!
    $sql = "INSERT INTO msgs (id, msg, dest, type, createtime, onlinemsg, username, pass, servername, serverhost, msgtaken, isstringmsg, msgencoding, msgdelivered)" .
        "VALUES ( nextval('hibernate_sequence'),  '" .  pg_escape_bytea($msg) . "',  '" . pg_escape_string($dest) . "',  '" .    pg_escape_bytea($type) . "',  " . time() . ",  " . $online2 . ", '" .    pg_escape_string($username) . "',   '" . pg_escape_string($pass) . "', '" . pg_escape_string($server) . "', '" . pg_escape_string($CFG->momo_msg_hostid) . "', false,true,'UTF-8', false)";
   // print($sql);
    // insert into messaging db
    _momo_msg_execute($sql);
}
?>